@@ -267,8 +267,11 @@ class Stream():
               except_targets=None,
               cross_junctions=False):
 
+        # We pass no target to build. Only to track. Passing build targets
+        # would fully load project configuration which might not be
+        # possible before tracking is done.
         _, elements = \
-            self._load(targets, targets,
+            self._load([], targets,
                        selection=selection, track_selection=selection,
                        except_targets=except_targets,
                        track_except_targets=except_targets,
@@ -434,7 +437,7 @@ class Stream():
 
         except BstError as e:
             raise StreamError("Error while staging dependencies into a sandbox"
-                              ": '{}'".format(e), reason=e.reason) from e
+                              ": '{}'".format(e), detail=e.detail, reason=e.reason) from e
 
     # workspace_open
     #
@@ -475,7 +478,7 @@ class Stream():
 
         # Check for workspace config
         workspace = workspaces.get_workspace(target._get_full_name())
-        if workspace:
+        if workspace and not force:
             raise StreamError("Workspace '{}' is already defined at: {}"
                               .format(target.name, workspace.path))
 
@@ -494,6 +497,10 @@ class Stream():
                               "fetch the latest version of the " +
                               "source.")
 
+        if workspace:
+            workspaces.delete_workspace(target._get_full_name())
+            workspaces.save_config()
+            shutil.rmtree(directory)
         try:
             os.makedirs(directory, exist_ok=True)
         except OSError as e:
@@ -651,7 +658,8 @@ class Stream():
     def source_bundle(self, target, directory, *,
                       track_first=False,
                       force=False,
-                      compression="gz"):
+                      compression="gz",
+                      except_targets=()):
 
         if track_first:
             track_targets = (target,)
@@ -660,6 +668,7 @@ class Stream():
 
         elements, track_elements = self._load((target,), track_targets,
                                               selection=PipelineSelection.ALL,
+                                              except_targets=except_targets,
                                               track_selection=PipelineSelection.ALL,
                                               fetch_subprojects=True)
 
@@ -817,6 +826,12 @@ class Stream():
     #
     # A convenience method for loading element lists
     #
+    # If `targets` is not empty used project configuration will be
+    # fully loaded. If `targets` is empty, tracking will still be
+    # resolved for elements in `track_targets`, but no build pipeline
+    # will be resolved. This is behavior is import for track() to
+    # not trigger full loading of project configuration.
+    #
     # Args:
     #    targets (list of str): Main targets to load
     #    track_targets (list of str): Tracking targets
@@ -864,7 +879,7 @@ class Stream():
         #
         # This can happen with `bst build --track`
         #
-        if not self._pipeline.targets_include(elements, track_elements):
+        if targets and not self._pipeline.targets_include(elements, track_elements):
             raise StreamError("Specified tracking targets that are not "
                               "within the scope of primary targets")
 
@@ -900,6 +915,10 @@ class Stream():
         for element in track_selected:
             element._schedule_tracking()
 
+        if not targets:
+            self._pipeline.resolve_elements(track_selected)
+            return [], track_selected
+
         # ArtifactCache.setup_remotes expects all projects to be fully loaded
         for project in self._context.get_projects():
             project.ensure_fully_loaded()
