@@ -123,6 +123,58 @@ def test_open_force(cli, tmpdir, datafiles, kind):
     result.assert_success()
 
 
+@pytest.mark.datafiles(DATA_DIR)
+@pytest.mark.parametrize("kind", repo_kinds)
+def test_open_force_open(cli, tmpdir, datafiles, kind):
+    element_name, project, workspace = open_workspace(cli, tmpdir, datafiles, kind, False)
+
+    # Assert the workspace dir exists
+    assert os.path.exists(workspace)
+
+    # Now open the workspace again with --force, this should happily succeed
+    result = cli.run(project=project, args=[
+        'workspace', 'open', '--force', element_name, workspace
+    ])
+    result.assert_success()
+
+
+@pytest.mark.datafiles(DATA_DIR)
+@pytest.mark.parametrize("kind", repo_kinds)
+def test_open_force_different_workspace(cli, tmpdir, datafiles, kind):
+    element_name, project, workspace = open_workspace(cli, tmpdir, datafiles, kind, False, "-alpha")
+
+    # Assert the workspace dir exists
+    assert os.path.exists(workspace)
+
+    hello_path = os.path.join(workspace, 'usr', 'bin', 'hello')
+    hello1_path = os.path.join(workspace, 'usr', 'bin', 'hello1')
+
+    tmpdir = os.path.join(str(tmpdir), "-beta")
+    shutil.move(hello_path, hello1_path)
+    element_name2, project2, workspace2 = open_workspace(cli, tmpdir, datafiles, kind, False, "-beta")
+
+    # Assert the workspace dir exists
+    assert os.path.exists(workspace2)
+
+    # Assert that workspace 1 contains the modified file
+    assert os.path.exists(hello1_path)
+
+    # Assert that workspace 2 contains the unmodified file
+    assert os.path.exists(os.path.join(workspace2, 'usr', 'bin', 'hello'))
+
+    # Now open the workspace again with --force, this should happily succeed
+    result = cli.run(project=project, args=[
+        'workspace', 'open', '--force', element_name2, workspace
+    ])
+
+    # Assert that the file in workspace 1 has been replaced
+    # With the file from workspace 2
+    assert os.path.exists(hello_path)
+    assert not os.path.exists(hello1_path)
+
+    result.assert_success()
+
+
 @pytest.mark.datafiles(DATA_DIR)
 @pytest.mark.parametrize("kind", repo_kinds)
 def test_close(cli, tmpdir, datafiles, kind):
